@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 Vincent Bernat <bernat@luffy.cx>
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -401,7 +401,18 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
 #define LLDP_SNMP_LOCAL_DOT3_AGG_STATUS 9
 #define LLDP_SNMP_LOCAL_DOT3_AGG_ID 10
 #define LLDP_SNMP_LOCAL_DOT3_MFS 11
-#define LLDP_SNMP_LOCAL_DOT1_PVID 12
+#define LLDP_SNMP_LOCAL_DOT3_POWER_DEVICETYPE 12
+#define LLDP_SNMP_LOCAL_DOT3_POWER_SUPPORT 13
+#define LLDP_SNMP_LOCAL_DOT3_POWER_ENABLED 14
+#define LLDP_SNMP_LOCAL_DOT3_POWER_PAIRCONTROL 15
+#define LLDP_SNMP_LOCAL_DOT3_POWER_PAIRS 16
+#define LLDP_SNMP_LOCAL_DOT3_POWER_CLASS 17
+#define LLDP_SNMP_LOCAL_DOT3_POWER_TYPE 18
+#define LLDP_SNMP_LOCAL_DOT3_POWER_SOURCE 19
+#define LLDP_SNMP_LOCAL_DOT3_POWER_PRIORITY 20
+#define LLDP_SNMP_LOCAL_DOT3_POWER_REQUESTED 21
+#define LLDP_SNMP_LOCAL_DOT3_POWER_ALLOCATED 22
+#define LLDP_SNMP_LOCAL_DOT1_PVID 23
 /* Remote ports */
 #define LLDP_SNMP_REMOTE_CIDSUBTYPE 1
 #define LLDP_SNMP_REMOTE_CID 2
@@ -419,7 +430,18 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
 #define LLDP_SNMP_REMOTE_DOT3_AGG_STATUS 14
 #define LLDP_SNMP_REMOTE_DOT3_AGG_ID 15
 #define LLDP_SNMP_REMOTE_DOT3_MFS 16
-#define LLDP_SNMP_REMOTE_DOT1_PVID 17
+#define LLDP_SNMP_REMOTE_DOT3_POWER_DEVICETYPE 17
+#define LLDP_SNMP_REMOTE_DOT3_POWER_SUPPORT 18
+#define LLDP_SNMP_REMOTE_DOT3_POWER_ENABLED 19
+#define LLDP_SNMP_REMOTE_DOT3_POWER_PAIRCONTROL 20
+#define LLDP_SNMP_REMOTE_DOT3_POWER_PAIRS 21
+#define LLDP_SNMP_REMOTE_DOT3_POWER_CLASS 22
+#define LLDP_SNMP_REMOTE_DOT3_POWER_TYPE 23
+#define LLDP_SNMP_REMOTE_DOT3_POWER_SOURCE 24
+#define LLDP_SNMP_REMOTE_DOT3_POWER_PRIORITY 25
+#define LLDP_SNMP_REMOTE_DOT3_POWER_REQUESTED 26
+#define LLDP_SNMP_REMOTE_DOT3_POWER_ALLOCATED 27
+#define LLDP_SNMP_REMOTE_DOT1_PVID 28
 /* Local vlans */
 #define LLDP_SNMP_LOCAL_DOT1_VLANNAME 1
 /* Remote vlans */
@@ -1061,16 +1083,16 @@ agent_h_local_port(struct variable *vp, oid *name, size_t *length,
 		return (u_char *)hardware->h_lport.p_descr;
 #ifdef ENABLE_DOT3
         case LLDP_SNMP_LOCAL_DOT3_AUTONEG_SUPPORT:
-                long_ret = 2 - hardware->h_lport.p_autoneg_support;
+                long_ret = 2 - hardware->h_lport.p_macphy.autoneg_support;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_LOCAL_DOT3_AUTONEG_ENABLED:
-                long_ret = 2 - hardware->h_lport.p_autoneg_enabled;
+                long_ret = 2 - hardware->h_lport.p_macphy.autoneg_enabled;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_LOCAL_DOT3_AUTONEG_ADVERTISED:
                 *var_len = 2;
-                return (u_char *)&hardware->h_lport.p_autoneg_advertised;
+                return (u_char *)&hardware->h_lport.p_macphy.autoneg_advertised;
         case LLDP_SNMP_LOCAL_DOT3_AUTONEG_MAU:
-                long_ret = hardware->h_lport.p_mau_type;
+                long_ret = hardware->h_lport.p_macphy.mau_type;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_LOCAL_DOT3_AGG_STATUS:
                 bit = swap_bits((hardware->h_lport.p_aggregid > 0) ? 3 : 0);
@@ -1082,6 +1104,83 @@ agent_h_local_port(struct variable *vp, oid *name, size_t *length,
 	case LLDP_SNMP_LOCAL_DOT3_MFS:
 		long_ret = hardware->h_lport.p_mfs;
 		return (u_char *)&long_ret;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_DEVICETYPE:
+		if (hardware->h_lport.p_power.devicetype) {
+			long_ret = (hardware->h_lport.p_power.devicetype ==
+			    LLDP_DOT3_POWER_PSE)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_SUPPORT:
+		if (hardware->h_lport.p_power.devicetype) {
+			long_ret = (hardware->h_lport.p_power.supported)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_ENABLED:
+		if (hardware->h_lport.p_power.devicetype) {
+			long_ret = (hardware->h_lport.p_power.enabled)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_PAIRCONTROL:
+		if (hardware->h_lport.p_power.devicetype) {
+			long_ret = (hardware->h_lport.p_power.paircontrol)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_PAIRS:
+		if (hardware->h_lport.p_power.devicetype) {
+			long_ret = hardware->h_lport.p_power.pairs;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_CLASS:
+		if (hardware->h_lport.p_power.devicetype && hardware->h_lport.p_power.class) {
+			long_ret = hardware->h_lport.p_power.class;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_TYPE:
+		if (hardware->h_lport.p_power.devicetype &&
+		    hardware->h_lport.p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			*var_len = 1;
+			bit = (((hardware->h_lport.p_power.powertype ==
+				    LLDP_DOT3_POWER_8023AT_TYPE1)?1:0) << 7) |
+			    (((hardware->h_lport.p_power.devicetype ==
+				    LLDP_DOT3_POWER_PSE)?0:1) << 6);
+			return (u_char *)&bit;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_SOURCE:
+		if (hardware->h_lport.p_power.devicetype &&
+		    hardware->h_lport.p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			*var_len = 1;
+			bit = swap_bits(hardware->h_lport.p_power.source%(1<<2));
+			return (u_char *)&bit;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_PRIORITY:
+		if (hardware->h_lport.p_power.devicetype &&
+		    hardware->h_lport.p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			long_ret = hardware->h_lport.p_power.priority;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_REQUESTED:
+		if (hardware->h_lport.p_power.devicetype &&
+		    hardware->h_lport.p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			long_ret = hardware->h_lport.p_power.requested;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_LOCAL_DOT3_POWER_ALLOCATED:
+		if (hardware->h_lport.p_power.devicetype &&
+		    hardware->h_lport.p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			long_ret = hardware->h_lport.p_power.allocated;
+			return (u_char *)&long_ret;
+		}
+		break;
 #endif
 #ifdef ENABLE_DOT1
 	case LLDP_SNMP_LOCAL_DOT1_PVID:
@@ -1091,7 +1190,10 @@ agent_h_local_port(struct variable *vp, oid *name, size_t *length,
 	default:
 		break;
         }
-        return NULL;
+	if (!exact && (name[*length-1] < MAX_SUBID))
+		return agent_h_local_port(vp, name, length,
+		    exact, var_len, write_method);
+	return NULL;
 }
 
 #ifdef ENABLE_DOT1
@@ -1180,16 +1282,16 @@ agent_h_remote_port(struct variable *vp, oid *name, size_t *length,
 		return (u_char *)&bit;
 #ifdef ENABLE_DOT3
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_SUPPORT:
-                long_ret = 2 - port->p_autoneg_support;
+                long_ret = 2 - port->p_macphy.autoneg_support;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_ENABLED:
-                long_ret = 2 - port->p_autoneg_enabled;
+                long_ret = 2 - port->p_macphy.autoneg_enabled;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_ADVERTISED:
                 *var_len = 2;
-                return (u_char *)&port->p_autoneg_advertised;
+                return (u_char *)&port->p_macphy.autoneg_advertised;
         case LLDP_SNMP_REMOTE_DOT3_AUTONEG_MAU:
-                long_ret = port->p_mau_type;
+                long_ret = port->p_macphy.mau_type;
                 return (u_char *)&long_ret;
         case LLDP_SNMP_REMOTE_DOT3_AGG_STATUS:
                 bit = swap_bits((port->p_aggregid > 0) ? 3 : 0);
@@ -1201,6 +1303,82 @@ agent_h_remote_port(struct variable *vp, oid *name, size_t *length,
         case LLDP_SNMP_REMOTE_DOT3_MFS:
                 long_ret = port->p_mfs;
                 return (u_char *)&long_ret;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_DEVICETYPE:
+		if (port->p_power.devicetype) {
+			long_ret = (port->p_power.devicetype == LLDP_DOT3_POWER_PSE)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_SUPPORT:
+		if (port->p_power.devicetype) {
+			long_ret = (port->p_power.supported)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_ENABLED:
+		if (port->p_power.devicetype) {
+			long_ret = (port->p_power.enabled)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_PAIRCONTROL:
+		if (port->p_power.devicetype) {
+			long_ret = (port->p_power.paircontrol)?1:2;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_PAIRS:
+		if (port->p_power.devicetype) {
+			long_ret = port->p_power.pairs;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_CLASS:
+		if (port->p_power.devicetype && port->p_power.class) {
+			long_ret = port->p_power.class;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_TYPE:
+		if (port->p_power.devicetype &&
+		    port->p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			*var_len = 1;
+			bit = (((port->p_power.powertype ==
+				    LLDP_DOT3_POWER_8023AT_TYPE1)?1:0) << 7) |
+			    (((port->p_power.devicetype ==
+				    LLDP_DOT3_POWER_PSE)?0:1) << 6);
+			return (u_char *)&bit;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_SOURCE:
+		if (port->p_power.devicetype &&
+		    port->p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			*var_len = 1;
+			bit = swap_bits(port->p_power.source%(1<<2));
+			return (u_char *)&bit;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_PRIORITY:
+		if (port->p_power.devicetype &&
+		    port->p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			long_ret = port->p_power.priority;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_REQUESTED:
+		if (port->p_power.devicetype &&
+		    port->p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			long_ret = port->p_power.requested;
+			return (u_char *)&long_ret;
+		}
+		break;
+	case LLDP_SNMP_REMOTE_DOT3_POWER_ALLOCATED:
+		if (port->p_power.devicetype &&
+		    port->p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			long_ret = port->p_power.allocated;
+			return (u_char *)&long_ret;
+		}
+		break;
 #endif
 #ifdef ENABLE_DOT1
         case LLDP_SNMP_REMOTE_DOT1_PVID:
@@ -1210,7 +1388,10 @@ agent_h_remote_port(struct variable *vp, oid *name, size_t *length,
 	default:
 		break;
         }
-        return NULL;
+	if (!exact && (name[*length-1] < MAX_SUBID))
+		return agent_h_remote_port(vp, name, length,
+		    exact, var_len, write_method);
+	return NULL;
 }
 
 static u_char*
@@ -1338,6 +1519,28 @@ static struct variable8 lldp_vars[] = {
          {1, 5, 4623, 1, 2, 1, 1, 3}},
         {LLDP_SNMP_LOCAL_DOT3_AUTONEG_MAU, ASN_INTEGER, RONLY, agent_h_local_port, 8,
          {1, 5, 4623, 1, 2, 1, 1, 4}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_DEVICETYPE, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 1}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_SUPPORT, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 2}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_ENABLED, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 3}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_PAIRCONTROL, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 4}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_PAIRS, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 5}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_CLASS, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 6}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_TYPE, ASN_OCTET_STR, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 7}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_SOURCE, ASN_OCTET_STR, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 8}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_PRIORITY, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 9}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_REQUESTED, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 10}},
+	{LLDP_SNMP_LOCAL_DOT3_POWER_ALLOCATED, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+	 {1, 5, 4623, 1, 2, 2, 1, 11}},
         {LLDP_SNMP_LOCAL_DOT3_AGG_STATUS, ASN_OCTET_STR, RONLY, agent_h_local_port, 8,
          {1, 5, 4623, 1, 2, 3, 1, 1}},
         {LLDP_SNMP_LOCAL_DOT3_AGG_ID, ASN_INTEGER, RONLY, agent_h_local_port, 8,
@@ -1368,6 +1571,28 @@ static struct variable8 lldp_vars[] = {
          {1, 5, 4623, 1, 3, 1, 1, 3}},
         {LLDP_SNMP_REMOTE_DOT3_AUTONEG_MAU, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
          {1, 5, 4623, 1, 3, 1, 1, 4}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_DEVICETYPE, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 1}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_SUPPORT, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 2}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_ENABLED, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 3}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_PAIRCONTROL, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 4}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_PAIRS, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 5}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_CLASS, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 6}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_TYPE, ASN_OCTET_STR, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 7}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_SOURCE, ASN_OCTET_STR, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 8}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_PRIORITY, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 9}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_REQUESTED, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 10}},
+	{LLDP_SNMP_REMOTE_DOT3_POWER_ALLOCATED, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+	 {1, 5, 4623, 1, 3, 2, 1, 11}},
         {LLDP_SNMP_REMOTE_DOT3_AGG_STATUS, ASN_OCTET_STR, RONLY, agent_h_remote_port, 8,
          {1, 5, 4623, 1, 3, 3, 1, 1}},
         {LLDP_SNMP_REMOTE_DOT3_AGG_ID, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
