@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 Vincent Bernat <bernat@luffy.cx>
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -242,6 +242,46 @@ static const struct value_string operational_mau_type_values[] = {
 	{ 53,	"1000BasePX20U - One single-mode fiber EPON ONU, 20km" },
 	{ 0, NULL }
 };
+
+static const struct value_string port_dot3_power_devicetype_map[] = {
+	{ LLDP_DOT3_POWER_PSE, "PSE" },
+	{ LLDP_DOT3_POWER_PD,  "PD" },
+	{ 0, NULL }
+};
+
+static const struct value_string port_dot3_power_pairs_map[] = {
+	{ LLDP_DOT3_POWERPAIRS_SIGNAL, "signal" },
+	{ LLDP_DOT3_POWERPAIRS_SPARE,  "spare" },
+	{ 0, NULL }
+};
+
+static const struct value_string port_dot3_power_class_map[] = {
+	{ 1, "class 0" },
+	{ 2, "class 1" },
+	{ 3, "class 2" },
+	{ 4, "class 3" },
+	{ 5, "class 4" },
+	{ 0, NULL }
+};
+
+static const struct value_string port_dot3_power_pse_source_map[] = {
+	{ LLDP_DOT3_POWER_SOURCE_BOTH, "PSE + Local" },
+	{ LLDP_DOT3_POWER_SOURCE_PSE, "PSE" },
+	{ 0, NULL }
+};
+
+static const struct value_string port_dot3_power_pd_source_map[] = {
+	{ LLDP_DOT3_POWER_SOURCE_BACKUP, "Backup source" },
+	{ LLDP_DOT3_POWER_SOURCE_PRIMARY, "Primary power source" },
+	{ 0, NULL }
+};
+
+static const struct value_string port_dot3_power_priority_map[] = {
+	{ LLDPMED_POW_PRIO_CRITICAL, "critical" },
+	{ LLDPMED_POW_PRIO_HIGH,     "high" },
+	{ LLDPMED_POW_PRIO_LOW,      "low" },
+	{ 0, NULL },
+};
 #endif
 
 static const struct value_string chassis_capability_map[] = {
@@ -791,14 +831,14 @@ display_chassis(struct writer * w, struct lldpd_chassis *chassis)
 static void
 display_autoneg(struct writer * w, struct lldpd_port *port, int bithd, int bitfd, char *desc)
 {
-	if (!((port->p_autoneg_advertised & bithd) ||
-		(port->p_autoneg_advertised & bitfd)))
+	if (!((port->p_macphy.autoneg_advertised & bithd) ||
+		(port->p_macphy.autoneg_advertised & bitfd)))
 		return;
 
 	tag_start(w, "advertised", "Adv");
 	tag_attr(w, "type", "", desc);
-	tag_attr(w, "hd", "HD", (port->p_autoneg_advertised & bithd)?"yes":"no");
-	tag_attr(w, "fd", "FD", (port->p_autoneg_advertised)?"yes":"no");
+	tag_attr(w, "hd", "HD", (port->p_macphy.autoneg_advertised & bithd)?"yes":"no");
+	tag_attr(w, "fd", "FD", (port->p_macphy.autoneg_advertised)?"yes":"no");
 	tag_end (w);
 }
 #endif
@@ -852,13 +892,15 @@ display_port(struct writer * w, struct lldpd_port *port)
 		tag_datatag(w, "aggregation", " Port is aggregated. PortAggregID",
 		            u2str(port->p_aggregid));
 
-	if (port->p_autoneg_support || port->p_autoneg_enabled ||
-	    port->p_mau_type) {
+	if (port->p_macphy.autoneg_support || port->p_macphy.autoneg_enabled ||
+	    port->p_macphy.mau_type) {
 		tag_start(w, "auto-negotiation", "PMD autoneg");
-		tag_attr (w, "supported", "supported", port->p_autoneg_support?"yes":"no");
-		tag_attr (w, "enabled", "enabled", port->p_autoneg_enabled?"yes":"no");
+		tag_attr (w, "supported", "supported",
+		    port->p_macphy.autoneg_support?"yes":"no");
+		tag_attr (w, "enabled", "enabled",
+		    port->p_macphy.autoneg_enabled?"yes":"no");
 
-		if (port->p_autoneg_enabled) {
+		if (port->p_macphy.autoneg_enabled) {
 			display_autoneg(w, port, LLDP_DOT3_LINK_AUTONEG_10BASE_T,
 			    LLDP_DOT3_LINK_AUTONEG_10BASET_FD,
 			    "10Base-T");
@@ -876,7 +918,58 @@ display_port(struct writer * w, struct lldpd_port *port)
 			    "1000Base-T");
 		}
 		tag_datatag(w, "current", "MAU oper type",
-			map_lookup(operational_mau_type_values, port->p_mau_type));
+			map_lookup(operational_mau_type_values, port->p_macphy.mau_type));
+		tag_end(w);
+	}
+	if (port->p_power.devicetype) {
+		tag_start(w, "power", "MDI Power");
+		tag_attr(w, "supported", "supported",
+		    port->p_power.supported?"yes":"no");
+		tag_attr(w, "enabled", "enabled",
+		    port->p_power.enabled?"yes":"no");
+		tag_attr(w, "paircontrol", "pair control",
+		    port->p_power.paircontrol?"yes":"no");
+		tag_start(w, "device-type", "Device type");
+		tag_data(w, map_lookup(port_dot3_power_devicetype_map,
+			port->p_power.devicetype));
+		tag_end(w);
+		tag_start(w, "pairs", "Power pairs");
+		tag_data(w, map_lookup(port_dot3_power_pairs_map,
+			port->p_power.pairs));
+		tag_end(w);
+		tag_start(w, "class", "Class");
+		tag_data(w, map_lookup(port_dot3_power_class_map,
+			port->p_power.class));
+		tag_end(w);
+
+		/* 802.3at */
+		if (port->p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			tag_start(w, "power-type", "Power type");
+			tag_data(w, u2str(port->p_power.powertype));
+			tag_end(w);
+
+			tag_start(w, "source", "Power Source");
+			tag_data(w, map_lookup(
+				    (port->p_power.devicetype == LLDP_DOT3_POWER_PSE)?
+					port_dot3_power_pse_source_map:
+					port_dot3_power_pd_source_map,
+					port->p_power.source));
+			tag_end(w);
+
+			tag_start(w, "priority", "Power Priority");
+			tag_data(w, map_lookup(port_dot3_power_priority_map,
+				port->p_power.priority));
+			tag_end(w);
+
+			tag_start(w, "requested", "PD requested power Value");
+			tag_data(w, u2str(port->p_power.requested * 100));
+			tag_end(w);
+
+			tag_start(w, "allocated", "PSE allocated power Value");
+			tag_data(w, u2str(port->p_power.allocated * 100));
+			tag_end(w);
+		}
+
 		tag_end(w);
 	}
 #endif
