@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 Vincent Bernat <bernat@luffy.cx>
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -527,45 +527,45 @@ iface_macphy(struct lldpd_hardware *hardware)
 		{0,0}};
 
 	if (priv_ethtool(hardware->h_ifname, &ethc) == 0) {
-		port->p_autoneg_support = (ethc.supported & SUPPORTED_Autoneg) ? 1 : 0;
-		port->p_autoneg_enabled = (ethc.autoneg == AUTONEG_DISABLE) ? 0 : 1;
+		port->p_macphy.autoneg_support = (ethc.supported & SUPPORTED_Autoneg) ? 1 : 0;
+		port->p_macphy.autoneg_enabled = (ethc.autoneg == AUTONEG_DISABLE) ? 0 : 1;
 		for (j=0; advertised_ethtool_to_rfc3636[j][0]; j++) {
 			if (ethc.advertising & advertised_ethtool_to_rfc3636[j][0])
-				port->p_autoneg_advertised |= 
+				port->p_macphy.autoneg_advertised |= 
 				    advertised_ethtool_to_rfc3636[j][1];
 		}
 		switch (ethc.speed) {
 		case SPEED_10:
-			port->p_mau_type = (ethc.duplex == DUPLEX_FULL) ? \
+			port->p_macphy.mau_type = (ethc.duplex == DUPLEX_FULL) ? \
 			    LLDP_DOT3_MAU_10BASETFD : LLDP_DOT3_MAU_10BASETHD;
-			if (ethc.port == PORT_BNC) port->p_mau_type = LLDP_DOT3_MAU_10BASE2;
+			if (ethc.port == PORT_BNC) port->p_macphy.mau_type = LLDP_DOT3_MAU_10BASE2;
 			if (ethc.port == PORT_FIBRE)
-				port->p_mau_type = (ethc.duplex == DUPLEX_FULL) ? \
+				port->p_macphy.mau_type = (ethc.duplex == DUPLEX_FULL) ? \
 				    LLDP_DOT3_MAU_10BASEFLDF : LLDP_DOT3_MAU_10BASEFLHD;
 			break;
 		case SPEED_100:
-			port->p_mau_type = (ethc.duplex == DUPLEX_FULL) ? \
+			port->p_macphy.mau_type = (ethc.duplex == DUPLEX_FULL) ? \
 			    LLDP_DOT3_MAU_100BASETXFD : LLDP_DOT3_MAU_100BASETXHD;
 			if (ethc.port == PORT_BNC)
-				port->p_mau_type = (ethc.duplex == DUPLEX_FULL) ? \
+				port->p_macphy.mau_type = (ethc.duplex == DUPLEX_FULL) ? \
 				    LLDP_DOT3_MAU_100BASET2DF : LLDP_DOT3_MAU_100BASET2HD;
 			if (ethc.port == PORT_FIBRE)
-				port->p_mau_type = (ethc.duplex == DUPLEX_FULL) ? \
+				port->p_macphy.mau_type = (ethc.duplex == DUPLEX_FULL) ? \
 				    LLDP_DOT3_MAU_100BASEFXFD : LLDP_DOT3_MAU_100BASEFXHD;
 			break;
 		case SPEED_1000:
-			port->p_mau_type = (ethc.duplex == DUPLEX_FULL) ? \
+			port->p_macphy.mau_type = (ethc.duplex == DUPLEX_FULL) ? \
 			    LLDP_DOT3_MAU_1000BASETFD : LLDP_DOT3_MAU_1000BASETHD;
 			if (ethc.port == PORT_FIBRE)
-				port->p_mau_type = (ethc.duplex == DUPLEX_FULL) ? \
+				port->p_macphy.mau_type = (ethc.duplex == DUPLEX_FULL) ? \
 				    LLDP_DOT3_MAU_1000BASEXFD : LLDP_DOT3_MAU_1000BASEXHD;
 			break;
 		case SPEED_10000:
-			port->p_mau_type = (ethc.port == PORT_FIBRE) ?	\
+			port->p_macphy.mau_type = (ethc.port == PORT_FIBRE) ?	\
 					LLDP_DOT3_MAU_10GIGBASEX : LLDP_DOT3_MAU_10GIGBASER;
 			break;
 		}
-		if (ethc.port == PORT_AUI) port->p_mau_type = LLDP_DOT3_MAU_AUI;
+		if (ethc.port == PORT_AUI) port->p_macphy.mau_type = LLDP_DOT3_MAU_AUI;
 	}
 #endif
 }
@@ -686,7 +686,6 @@ lldpd_ifh_eth(struct lldpd *cfg, struct ifaddrs *ifap)
 {
 	struct ifaddrs *ifa;
 	struct lldpd_hardware *hardware;
-	struct lldpd_port *port;
 
 	for (ifa = ifap; ifa != NULL; ifa = ifa->ifa_next) {
 		if (!iface_minimal_checks(cfg, ifa))
@@ -715,7 +714,6 @@ lldpd_ifh_eth(struct lldpd *cfg, struct ifaddrs *ifap)
 			lldpd_port_cleanup(cfg, &hardware->h_lport, 0);
 		}
 
-		port = &hardware->h_lport;
 		hardware->h_flags = ifa->ifa_flags; /* Should be non-zero */
 		ifa->ifa_flags = 0;		    /* Future handlers
 						       don't have to
@@ -737,6 +735,41 @@ lldpd_ifh_eth(struct lldpd *cfg, struct ifaddrs *ifap)
 	}
 }
 
+void
+lldpd_ifh_whitelist(struct lldpd *cfg, struct ifaddrs *ifap)
+{
+	struct ifaddrs *ifa;
+	char *interfaces = NULL;
+	char *pattern;
+
+	if (!cfg->g_interfaces)
+		return;
+	if ((interfaces = strdup(cfg->g_interfaces)) == NULL) {
+		LLOG_WARNX("unable to allocate memory");
+		return;
+	}
+
+	for (ifa = ifap; ifa != NULL; ifa = ifa->ifa_next) {
+		if (ifa->ifa_flags == 0) continue; /* Already handled by someone else */
+		strcpy(interfaces, cfg->g_interfaces); /* Restore our list of interfaces */
+		pattern = strtok(interfaces, ",");
+		while (pattern != NULL) {
+			if (fnmatch(pattern, ifa->ifa_name, 0) == 0) {
+				/* This interface is whitelisted */
+				break;
+			}
+			pattern = strtok(NULL, ",");
+		}
+		if (pattern == NULL) {
+			/* This interface was not found. We flag it. */
+			LLOG_DEBUG("blacklist %s", ifa->ifa_name);
+			ifa->ifa_flags = 0;
+		}
+	}
+
+	free(interfaces);
+}
+
 static int
 iface_bond_init(struct lldpd *cfg, struct lldpd_hardware *hardware)
 {
