@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 Vincent Bernat <bernat@luffy.cx>
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -183,10 +183,10 @@ lldp_send(struct lldpd *global,
 	      POKE_START_LLDP_TLV(LLDP_TLV_ORG) &&
 	      POKE_BYTES(dot3, sizeof(dot3)) &&
 	      POKE_UINT8(LLDP_TLV_DOT3_MAC) &&
-	      POKE_UINT8(port->p_autoneg_support |
-			 (port->p_autoneg_enabled << 1)) &&
-	      POKE_UINT16(port->p_autoneg_advertised) &&
-	      POKE_UINT16(port->p_mau_type) &&
+	      POKE_UINT8(port->p_macphy.autoneg_support |
+			 (port->p_macphy.autoneg_enabled << 1)) &&
+	      POKE_UINT16(port->p_macphy.autoneg_advertised) &&
+	      POKE_UINT16(port->p_macphy.mau_type) &&
 	      POKE_END_LLDP_TLV))
 		goto toobig;
 
@@ -198,6 +198,37 @@ lldp_send(struct lldpd *global,
 	      POKE_UINT16(port->p_mfs) &&
 	      POKE_END_LLDP_TLV))
 		goto toobig;
+	/* Power */
+	if (port->p_power.devicetype) {
+		if (!(
+		      POKE_START_LLDP_TLV(LLDP_TLV_ORG) &&
+		      POKE_BYTES(dot3, sizeof(dot3)) &&
+		      POKE_UINT8(LLDP_TLV_DOT3_POWER) &&
+		      POKE_UINT8((
+				  (((2 - port->p_power.devicetype)    %(1<< 1))<<0) |
+				  (( port->p_power.supported          %(1<< 1))<<1) |
+				  (( port->p_power.enabled            %(1<< 1))<<2) |
+				  (( port->p_power.paircontrol        %(1<< 1))<<3))) &&
+		      POKE_UINT8(port->p_power.pairs) &&
+		      POKE_UINT8(port->p_power.class)))
+			goto toobig;
+		/* 802.3at */
+		if (port->p_power.powertype != LLDP_DOT3_POWER_8023AT_OFF) {
+			if (!(
+			      POKE_UINT8((
+					  (((port->p_power.powertype ==
+					      LLDP_DOT3_POWER_8023AT_TYPE1)?1:0) << 7) |
+					   (((port->p_power.devicetype ==
+					      LLDP_DOT3_POWER_PSE)?0:1) << 6) |
+					   ((port->p_power.source   %(1<< 2))<<4) |
+					   ((port->p_power.priority %(1<< 2))<<0))) &&
+			      POKE_UINT16(port->p_power.requested) &&
+			      POKE_UINT16(port->p_power.allocated)))
+				goto toobig;
+		}
+		if (!(POKE_END_LLDP_TLV))
+			goto toobig;
+	}
 #endif
 
 #ifdef ENABLE_LLDPMED
@@ -565,14 +596,14 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 				switch (tlv_subtype) {
 				case LLDP_TLV_DOT3_MAC:
 					CHECK_TLV_SIZE(9, "MAC/PHY");
-					port->p_autoneg_support = PEEK_UINT8;
-					port->p_autoneg_enabled =
-					    port->p_autoneg_support && 0x2;
-					port->p_autoneg_support =
-					    port->p_autoneg_support && 0x1;
-					port->p_autoneg_advertised =
+					port->p_macphy.autoneg_support = PEEK_UINT8;
+					port->p_macphy.autoneg_enabled =
+					    port->p_macphy.autoneg_support & 0x2;
+					port->p_macphy.autoneg_support =
+					    port->p_macphy.autoneg_support & 0x1;
+					port->p_macphy.autoneg_advertised =
 					    PEEK_UINT16;
-					port->p_mau_type = PEEK_UINT16;
+					port->p_macphy.mau_type = PEEK_UINT16;
 					break;
 				case LLDP_TLV_DOT3_LA:
 					CHECK_TLV_SIZE(9, "Link aggregation");
@@ -583,6 +614,37 @@ lldp_decode(struct lldpd *cfg, char *frame, int s,
 					CHECK_TLV_SIZE(6, "MFS");
 					port->p_mfs = PEEK_UINT16;
 					break;
+				case LLDP_TLV_DOT3_POWER:
+					CHECK_TLV_SIZE(7, "Power");
+					port->p_power.devicetype = PEEK_UINT8;
+					port->p_power.supported =
+						port->p_power.devicetype & 0x2;
+					port->p_power.enabled =
+						port->p_power.devicetype & 0x4;
+					port->p_power.paircontrol =
+						port->p_power.devicetype & 0x8;
+					port->p_power.devicetype =
+						(port->p_power.devicetype & 0x1)?
+						LLDP_DOT3_POWER_PSE:LLDP_DOT3_POWER_PD;
+					port->p_power.pairs = PEEK_UINT8;
+					port->p_power.class = PEEK_UINT8;
+					/* 802.3at? */
+					if (tlv_size >= 12) {
+						port->p_power.powertype = PEEK_UINT8;
+						port->p_power.source =
+						    (port->p_power.powertype & (1<<5 | 1<<4)) >> 4;
+						port->p_power.priority =
+						    (port->p_power.powertype & (1<<1 | 1<<0));
+						port->p_power.powertype =
+						    (port->p_power.powertype & (1<<7))?
+						    LLDP_DOT3_POWER_8023AT_TYPE1:
+						    LLDP_DOT3_POWER_8023AT_TYPE2;
+						port->p_power.requested = PEEK_UINT16;
+						port->p_power.allocated = PEEK_UINT16;
+					} else
+						port->p_power.powertype =
+						    LLDP_DOT3_POWER_8023AT_OFF;
+					break;
 				default:
 					/* Unknown Dot3 TLV, ignore it */
 					hardware->h_rx_unrecognized_cnt++;
