@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 Vincent Bernat <bernat@luffy.cx>
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -41,7 +41,7 @@ get_interfaces(int s, struct interfaces *ifs);
 extern void
 display_interfaces(int s, const char * fmt, int argc, char *argv[]);
 
-#define LLDPCTL_ARGS "hdf:L:P:O:"
+#define LLDPCTL_ARGS "hdf:L:P:O:o:"
 
 static void
 usage(void)
@@ -62,6 +62,10 @@ usage(void)
 	fprintf(stderr, "-O poe      Enable the trabsmission of LLDP-MED POE-MDI TLV\n");
 	fprintf(stderr, "            for the given interfaces.\n");
 #endif
+#ifdef ENABLE_DOT3
+	fprintf(stderr, "-o poe      Enable the trabsmission of Dot3 POE-MDI TLV\n");
+	fprintf(stderr, "            for the given interfaces.\n");
+#endif
 
 	fprintf(stderr, "\n");
 
@@ -453,7 +457,141 @@ lldpd_parse_power(struct lldpd_port *port, const char *poe)
 	LLOG_WARNX("The format POE-MDI is invalid (%s)", poe);
 	return -1;
 }
+#endif
 
+#ifdef ENABLE_DOT3
+static int
+lldpd_parse_dot3_power(struct lldpd_port *port, const char *poe)
+{
+	const char *e;
+	struct lldpd_dot3_power target;
+
+	if (strlen(poe) == 0)
+		return 0;
+	e = poe;
+	memset(&target, 0, sizeof(target));
+
+	/* Device type */
+	if (!strncmp(e, "PD", 2))
+		target.devicetype = LLDP_DOT3_POWER_PD;
+	else if (!strncmp(e, "PSE", 3))
+		target.devicetype = LLDP_DOT3_POWER_PSE;
+	else {
+		LLOG_WARNX("Device type should be either 'PD' or 'PSE'.");
+		goto invalid_dot3_poe;
+	}
+
+	/* Supported */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected power support.");
+		goto invalid_dot3_poe;
+	}
+	target.supported = atoi(++e);
+	if (target.supported > 1) {
+		LLOG_WARNX("Power support should be 1 or 0, not %d", target.supported);
+		goto invalid_dot3_poe;
+	}
+
+	/* Enabled */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected power ability.");
+		goto invalid_dot3_poe;
+	}
+	target.enabled = atoi(++e);
+	if (target.enabled > 1) {
+		LLOG_WARNX("Power ability should be 1 or 0, not %d", target.enabled);
+		goto invalid_dot3_poe;
+	}
+
+	/* Pair control */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected power pair control ability.");
+		goto invalid_dot3_poe;
+	}
+	target.paircontrol = atoi(++e);
+	if (target.paircontrol > 1) {
+		LLOG_WARNX("Power pair control ability should be 1 or 0, not %d",
+		    target.paircontrol);
+		goto invalid_dot3_poe;
+	}
+
+	/* Power pairs */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected power pairs.");
+		goto invalid_dot3_poe;
+	}
+	target.pairs = atoi(++e);
+	if (target.pairs < 1 || target.pairs > 2) {
+		LLOG_WARNX("Power pairs should be 1 or 2, not %d.", target.pairs);
+		goto invalid_dot3_poe;
+	}
+
+	/* Class */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected power class.");
+		goto invalid_dot3_poe;
+	}
+	target.class = atoi(++e);
+	if (target.class > 5) {
+		LLOG_WARNX("Power class out of range (%d).", target.class);
+		goto invalid_dot3_poe;
+	}
+	/* 802.3at */
+	if ((e = strchr(e, ':')) == NULL) {
+		target.powertype = LLDP_DOT3_POWER_8023AT_OFF;
+		goto no8023at;
+	}
+	/* 802.3at: Power type */
+	target.powertype = atoi(++e);
+	if ((target.powertype != LLDP_DOT3_POWER_8023AT_TYPE1) &&
+	    (target.powertype != LLDP_DOT3_POWER_8023AT_TYPE2)) {
+		LLOG_WARNX("Incorrect power type (%d).", target.powertype);
+		goto invalid_dot3_poe;
+	}
+	/* 802.3at: Source */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected power source.");
+		goto invalid_dot3_poe;
+	}
+	target.source = atoi(++e);
+	if (target.source > 3) {
+		LLOG_WARNX("Power source out of range (%d).", target.source);
+		goto invalid_dot3_poe;
+	}
+	/* 802.3at: priority */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected power priority.");
+		goto invalid_dot3_poe;
+	}
+	target.priority = atoi(++e);
+	if (target.priority > 3) {
+		LLOG_WARNX("Power priority out of range (%d).", target.priority);
+		goto invalid_dot3_poe;
+	}
+	/* 802.3at: requested */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected requested power value.");
+		goto invalid_dot3_poe;
+	}
+	target.requested = atoi(++e);
+	/* 802.3at: allocated */
+	if ((e = strchr(e, ':')) == NULL) {
+		LLOG_WARNX("Expected allocated power value.");
+		goto invalid_dot3_poe;
+	}
+	target.allocated = atoi(++e);
+
+ no8023at:
+	memcpy(&port->p_power, &target, sizeof(target));
+	return 0;
+
+ invalid_dot3_poe:
+	LLOG_WARNX("The format POE-MDI is invalid (%s)", poe);
+	return -1;
+}
+#endif
+
+#ifdef ENABLE_LLDPMED
 static void
 set_location(int s, int argc, char *argv[])
 {
@@ -505,7 +643,7 @@ set_location(int s, int argc, char *argv[])
 			fatalx("set_location: unable to receive answer");
 		if (h->hdr.type != HMSG_SET_LOCATION)
 			fatalx("set_location: unknown answer type received");
-		LLOG_INFO("Location set succesfully for %s", iff->name);
+		LLOG_INFO("Location set successfully for %s", iff->name);
 	}
 }
 
@@ -626,6 +764,62 @@ set_power(int s, int argc, char *argv[])
 }
 #endif
 
+#ifdef ENABLE_DOT3
+static void
+set_dot3_power(int s, int argc, char *argv[])
+{
+	int i, ch;
+	struct interfaces ifs;
+	struct lldpd_interface *iff;
+	struct lldpd_port port;
+	void *p;
+	struct hmsg *h;
+
+	if ((h = (struct hmsg *)malloc(MAX_HMSGSIZE)) == NULL)
+		fatal(NULL);
+
+	memset(&port, 0, sizeof(struct lldpd_port));
+	optind = 1;
+	while ((ch = getopt(argc, argv, LLDPCTL_ARGS)) != -1) {
+		switch (ch) {
+		case 'o':
+			if ((lldpd_parse_dot3_power(&port, optarg)) == -1)
+				fatalx("Incorrect POE-MDI.");
+			break;
+		}
+	}
+
+	get_interfaces(s, &ifs);
+	TAILQ_FOREACH(iff, &ifs, next) {
+		if (optind < argc) {
+			for (i = optind; i < argc; i++)
+				if (strncmp(argv[i], iff->name, IFNAMSIZ) == 0)
+					break;
+			if (i == argc)
+				continue;
+		}
+
+		ctl_msg_init(h, HMSG_SET_DOT3_POWER);
+		strlcpy((char *)&h->data, iff->name, IFNAMSIZ);
+		h->hdr.len += IFNAMSIZ;
+		p = (char*)&h->data + IFNAMSIZ;
+		if (ctl_msg_pack_structure(STRUCT_LLDPD_DOT3_POWER,
+					   &port.p_power,
+					   sizeof(struct lldpd_dot3_power), h, &p) == -1) {
+			LLOG_WARNX("set_dot3_power: Unable to set POE-MDI for %s", iff->name);
+			fatalx("aborting");
+		}
+		if (ctl_msg_send(s, h) == -1)
+			fatalx("set_dot3_power: unable to send request");
+		if (ctl_msg_recv(s, h) == -1)
+			fatalx("set_dot3_power: unable to receive answer");
+		if (h->hdr.type != HMSG_SET_DOT3_POWER)
+			fatalx("set_dot3_power: unknown answer type received");
+		LLOG_INFO("Dot3 POE-MDI successfully set for %s", iff->name);
+	}
+}
+#endif
+
 int
 main(int argc, char *argv[])
 {
@@ -634,6 +828,7 @@ main(int argc, char *argv[])
 #define ACTION_SET_LOCATION (1 << 0)
 #define ACTION_SET_POLICY   (1 << 1)
 #define ACTION_SET_POWER    (1 << 2)
+#define ACTION_SET_DOT3_POWER    (1 << 3)
 	int action = 0;
 	
 	/*
@@ -662,6 +857,14 @@ main(int argc, char *argv[])
 #else
 			fprintf(stderr, "LLDP-MED support is not built-in\n");
 			usage();
+#endif
+			break;
+		case 'o':
+#ifdef ENABLE_DOT3
+			action |= ACTION_SET_DOT3_POWER;
+#else
+			fprintf(stderr, "Dot3 support is not built-in\n");
+			usage();
 #endif
 			break;
 		default:
@@ -671,7 +874,7 @@ main(int argc, char *argv[])
 
 	log_init(debug, __progname);
 
-	if ( ( action != 0 ) && ( getuid() != geteuid() ) ) {
+	if ((action != 0) && (getuid() != 0)) {
 		fatalx("mere mortals may not do that, 'root' privileges are required.");
 	}
 	
@@ -685,6 +888,10 @@ main(int argc, char *argv[])
 		set_policy(s, argc, argv);
 	if (action & ACTION_SET_POWER)
 		set_power(s, argc, argv);
+#endif
+#ifdef ENABLE_DOT3
+	if (action & ACTION_SET_DOT3_POWER)
+		set_dot3_power(s, argc, argv);
 #endif
 	if (!action)
 		display_interfaces(s, fmt, argc, argv);
