@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 Vincent Bernat <bernat@luffy.cx>
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -82,12 +82,14 @@ static void		 lldpd_exit(void);
 static void		 lldpd_send_all(struct lldpd *);
 static void		 lldpd_recv_all(struct lldpd *);
 static void		 lldpd_hide_all(struct lldpd *);
+static void		 lldpd_hide_ports(struct lldpd *, struct lldpd_hardware *, int);
 static int		 lldpd_guess_type(struct lldpd *, char *, int);
 static void		 lldpd_decode(struct lldpd *, char *, int,
 			    struct lldpd_hardware *);
 static void		 lldpd_update_chassis(struct lldpd_chassis *,
 			    const struct lldpd_chassis *);
 static char 		*lldpd_get_lsb_release(void);
+static char		*lldpd_get_os_release(void);
 #ifdef ENABLE_LLDPMED
 static void		 lldpd_med(struct lldpd_chassis *);
 #endif
@@ -107,6 +109,7 @@ usage(void)
 	fprintf(stderr, "\n");
 
 	fprintf(stderr, "-d       Do not daemonize.\n");
+	fprintf(stderr, "-r       Receive-only mode\n");
 	fprintf(stderr, "-i       Disable LLDP-MED inventory TLV transmission.\n");
 	fprintf(stderr, "-k       Disable advertising of kernel release, version, machine.\n");
 	fprintf(stderr, "-S descr Override the default system description.\n");
@@ -551,6 +554,46 @@ lldpd_get_lsb_release() {
 	return NULL;
 }
 
+/* Same like lldpd_get_lsb_release but reads /etc/os-release for PRETTY_NAME=. */
+static char *
+lldpd_get_os_release() {
+	static char release[1024];
+
+	FILE *fp = fopen("/etc/os-release", "r");
+	if (!fp) {
+		LLOG_WARN("could not open /etc/os-release");
+		return NULL;
+	}
+
+	char line[1024];
+	char *key, *val;
+
+	while ((fgets(line, 1024, fp) != NULL)) {
+		key = strtok(line, "=");
+		val = strtok(NULL, "=");
+
+		if (strncmp(key, "PRETTY_NAME", 1024) == 0) {
+			strncpy(release, val, 1024);
+			break;
+		}
+	}
+	fclose(fp);
+
+	/* Remove trailing newline and all " in the string. */
+	char *ptr1 = release;
+	char *ptr2 = release;
+	while (*ptr1 != 0) {
+		if ((*ptr1 == '"') || (*ptr1 == '\n')) {
+			++ptr1;
+		} else {
+			*ptr2++ = *ptr1++;
+		}
+	}
+	*ptr2 = 0;
+
+	return release;
+}
+
 int
 lldpd_callback_add(struct lldpd *cfg, int fd, void(*fn)(CALLBACK_SIG), void *data)
 {
@@ -587,69 +630,102 @@ static void
 lldpd_hide_all(struct lldpd *cfg)
 {
 	struct lldpd_hardware *hardware;
+
+	if (!cfg->g_smart)
+		return;
+	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
+		if (cfg->g_smart & SMART_INCOMING_FILTER)
+			lldpd_hide_ports(cfg, hardware, SMART_INCOMING);
+		if (cfg->g_smart & SMART_OUTGOING_FILTER)
+			lldpd_hide_ports(cfg, hardware, SMART_OUTGOING);
+	}
+}
+
+static void
+lldpd_hide_ports(struct lldpd *cfg, struct lldpd_hardware *hardware, int mask) {
 	struct lldpd_port *port;
 	int protocols[LLDPD_MODE_MAX+1];
-	int i, j, found;
+	char buffer[256];
+	int i, j, k, found;
 	unsigned int min;
 
-	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
-		/* Compute the number of occurrences of each protocol */
-		for (i = 0; i <= LLDPD_MODE_MAX; i++)
-			protocols[i] = 0;
-		TAILQ_FOREACH(port, &hardware->h_rports, p_entries)
-			protocols[port->p_protocol]++;
+	/* Compute the number of occurrences of each protocol */
+	for (i = 0; i <= LLDPD_MODE_MAX; i++) protocols[i] = 0;
+	TAILQ_FOREACH(port, &hardware->h_rports, p_entries)
+		protocols[port->p_protocol]++;
 
-		/* Turn the protocols[] array into an array of
-		   enabled/disabled protocols. 1 means enabled, 0
-		   means disabled. */
-		min = (unsigned int)-1;
-		for (i = 0; i <= LLDPD_MODE_MAX; i++)
-			if (protocols[i] && (protocols[i] < min))
-				min = protocols[i];
+	/* Turn the protocols[] array into an array of
+	   enabled/disabled protocols. 1 means enabled, 0
+	   means disabled. */
+	min = (unsigned int)-1;
+	for (i = 0; i <= LLDPD_MODE_MAX; i++)
+		if (protocols[i] && (protocols[i] < min))
+			min = protocols[i];
+	found = 0;
+	for (i = 0; i <= LLDPD_MODE_MAX; i++)
+		if ((protocols[i] == min) && !found) {
+			/* If we need a tie breaker, we take
+			   the first protocol only */
+			if (cfg->g_smart & mask &
+			    (SMART_OUTGOING_ONE_PROTO | SMART_INCOMING_ONE_PROTO))
+				found = 1;
+			protocols[i] = 1;
+		} else protocols[i] = 0;
+
+	/* We set the p_hidden flag to 1 if the protocol is disabled */
+	TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+		if (mask == SMART_OUTGOING)
+			port->p_hidden_out = protocols[port->p_protocol]?0:1;
+		else
+			port->p_hidden_in = protocols[port->p_protocol]?0:1;
+	}
+
+	/* If we want only one neighbor, we take the first one */
+	if (cfg->g_smart & mask &
+	    (SMART_OUTGOING_ONE_NEIGH | SMART_INCOMING_ONE_NEIGH)) {
 		found = 0;
-		for (i = 0; i <= LLDPD_MODE_MAX; i++)
-			if ((protocols[i] == min) && !found) {
-				/* If we need a tie breaker, we take
-				   the first protocol only */
-				if (cfg->g_smart & SMART_FILTER_NO_TIE)
-					found = 1;
-				protocols[i] = 1;
-			} else protocols[i] = 0;
-
-		/* We set the p_hidden flag to 1 if the protocol is disabled */
-		TAILQ_FOREACH(port, &hardware->h_rports, p_entries)
-			port->p_hidden = protocols[port->p_protocol]?0:1;
-
-		/* If we want only one neighbor, we take the first one */
-		if (cfg->g_smart & SMART_FILTER_ONE_NEIGH) {
-			found = 0;
-			TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
-				if (!port->p_hidden) {
-					if (found)
-						port->p_hidden = 1;
-					else
-						found = 1;
-				}
-			}
-		}
-
-		/* Print a debug message summarizing the operation */
-		i = j = 0;
 		TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
-		    if (port->p_hidden) i++;
-		    j++;
-		}
-		if (i) {
-			LLOG_DEBUG("On %s, out of %d neighbors, %d are hidden",
-			    hardware->h_ifname, j, i);
-			for (i=0; protos[i].mode != 0; i++) {
-				if (protos[i].enabled)
-					LLOG_DEBUG("On %s, %s is %s",
-					    hardware->h_ifname, protos[i].name,
-					    protocols[protos[i].mode]?"enabled":"disabled");
+			if (mask == SMART_OUTGOING) {
+				if (found) port->p_hidden_out = 1;
+				if (!port->p_hidden_out)
+					found = 1;
+			}
+			if (mask == SMART_INCOMING) {
+				if (found) port->p_hidden_in = 1;
+				if (!port->p_hidden_in)
+					found = 1;
 			}
 		}
 	}
+
+	/* Print a debug message summarizing the operation */
+	for (i = 0; i <= LLDPD_MODE_MAX; i++) protocols[i] = 0;
+	k = j = 0;
+	TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
+		if (!(((mask == SMART_OUTGOING) && port->p_hidden_out) ||
+		      ((mask == SMART_INCOMING) && port->p_hidden_in))) {
+			k++;
+			protocols[port->p_protocol] = 1;
+		}
+		j++;
+	}
+	buffer[0] = '\0';
+	for (i=0; cfg->g_protocols[i].mode != 0; i++) {
+		if (cfg->g_protocols[i].enabled && protocols[cfg->g_protocols[i].mode]) {
+			if (strlen(buffer) +
+			    strlen(cfg->g_protocols[i].name) + 3 > sizeof(buffer)) {
+				/* Unlikely, our buffer is too small */
+				memcpy(buffer + sizeof(buffer) - 4, "...", 4);
+				break;
+			}
+			if (buffer[0])
+				strcat(buffer, ", ");
+			strcat(buffer, cfg->g_protocols[i].name);
+		}
+	}
+	LLOG_DEBUG("[%s] %s: %d visible neigh / %d. Protocols: %s.",
+		   (mask == SMART_OUTGOING)?"out filter":"in filter",
+		   hardware->h_ifname, k, j, buffer[0]?buffer:"(none)");
 }
 
 static void
@@ -742,6 +818,7 @@ lldpd_recv_all(struct lldpd *cfg)
 			}
 			hardware->h_rx_cnt++;
 			lldpd_decode(cfg, buffer, n, hardware);
+			lldpd_hide_all(cfg); /* Immediatly hide */
 			free(buffer);
 			break;
 		}
@@ -790,8 +867,8 @@ lldpd_send_all(struct lldpd *cfg)
 			TAILQ_FOREACH(port, &hardware->h_rports, p_entries) {
 				/* If this remote port is disabled, we don't
 				 * consider it */
-				if (port->p_hidden &&
-				    (cfg->g_smart & SMART_FILTER_EMISSION))
+				if (port->p_hidden_out &&
+				    (cfg->g_smart & SMART_OUTGOING_FILTER))
 					continue;
 				if (port->p_protocol ==
 				    cfg->g_protocols[i].mode) {
@@ -853,7 +930,7 @@ lldpd_update_localchassis(struct lldpd *cfg)
 			fatal("failed to set full system description");
         } else {
 	        if (cfg->g_advertise_version) {
-		        if (asprintf(&LOCAL_CHASSIS(cfg)->c_descr, "%s%s %s %s",
+		        if (asprintf(&LOCAL_CHASSIS(cfg)->c_descr, "%s %s %s %s",
 			        cfg->g_lsb_release?cfg->g_lsb_release:"",
 				un.sysname, un.release, un.machine)
                                 == -1)
@@ -867,9 +944,10 @@ lldpd_update_localchassis(struct lldpd *cfg)
 
 	/* Check forwarding */
 	if ((f = priv_open("/proc/sys/net/ipv4/ip_forward")) >= 0) {
-		if ((read(f, &status, 1) == 1) && (status == '1')) {
-			LOCAL_CHASSIS(cfg)->c_cap_enabled = LLDP_CAP_ROUTER;
-		}
+		if ((read(f, &status, 1) == 1) && (status == '1'))
+			LOCAL_CHASSIS(cfg)->c_cap_enabled |= LLDP_CAP_ROUTER;
+		else
+			LOCAL_CHASSIS(cfg)->c_cap_enabled &= ~LLDP_CAP_ROUTER;
 		close(f);
 	}
 #ifdef ENABLE_LLDPMED
@@ -902,6 +980,7 @@ lldpd_update_localports(struct lldpd *cfg)
 	struct ifaddrs *ifap;
 	struct lldpd_hardware *hardware;
 	lldpd_ifhandlers ifhs[] = {
+		lldpd_ifh_whitelist, /* Is the interface whitelisted? */
 		lldpd_ifh_bond,	/* Handle bond */
 		lldpd_ifh_eth,	/* Handle classic ethernet interfaces */
 #ifdef ENABLE_DOT1
@@ -951,10 +1030,9 @@ lldpd_loop(struct lldpd *cfg)
 	lldpd_update_localports(cfg);
 	lldpd_cleanup(cfg);
 	lldpd_update_localchassis(cfg);
-	lldpd_send_all(cfg);
+	if (!cfg->g_receiveonly)
+		lldpd_send_all(cfg);
 	lldpd_recv_all(cfg);
-	if (cfg->g_smart != SMART_NOFILTER)
-		lldpd_hide_all(cfg);
 }
 
 static void
@@ -984,6 +1062,42 @@ lldpd_exit()
 #endif /* USE_SNMP */
 }
 
+struct intint { int a; int b; };
+static const struct intint filters[] = {
+	{  0, 0 },
+	{  1, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_PROTO },
+	{  2, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO },
+	{  3, SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_PROTO },
+	{  4, SMART_INCOMING_FILTER | SMART_OUTGOING_FILTER },
+	{  5, SMART_INCOMING_FILTER },
+	{  6, SMART_OUTGOING_FILTER },
+	{  7, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO | SMART_INCOMING_ONE_NEIGH |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_PROTO },
+	{  8, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO | SMART_INCOMING_ONE_NEIGH },
+	{  9, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_NEIGH |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_PROTO },
+	{ 10, SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_NEIGH },
+	{ 11, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_NEIGH },
+	{ 12, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_NEIGH |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_NEIGH },
+	{ 13, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_NEIGH |
+	      SMART_OUTGOING_FILTER },
+	{ 14, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_NEIGH },
+	{ 15, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO |
+	      SMART_OUTGOING_FILTER },
+	{ 16, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO | SMART_INCOMING_ONE_NEIGH |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_NEIGH },
+	{ 17, SMART_INCOMING_FILTER | SMART_INCOMING_ONE_PROTO | SMART_INCOMING_ONE_NEIGH |
+	      SMART_OUTGOING_FILTER },
+	{ 18, SMART_INCOMING_FILTER |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_NEIGH },
+	{ 19, SMART_INCOMING_FILTER |
+	      SMART_OUTGOING_FILTER | SMART_OUTGOING_ONE_PROTO },
+	{ -1, 0 }
+};
+
 int
 lldpd_main(int argc, char *argv[])
 {
@@ -995,15 +1109,17 @@ lldpd_main(int argc, char *argv[])
 	char *agentx = NULL;	/* AgentX socket */
 #endif
 	char *mgmtp = NULL;
+	char *interfaces = NULL;
 	char *popt, opts[] = 
-		"H:hkdxX:m:p:M:S:i@                    ";
+		"H:hkrdxX:m:I:p:M:S:i@                    ";
 	int i, found, advertise_version = 1;
 #ifdef ENABLE_LLDPMED
 	int lldpmed = 0, noinventory = 0;
 #endif
         char *descr_override = NULL;
 	char *lsb_release = NULL;
-	int smart = SMART_FILTER_NO_TIE | SMART_FILTER_EMISSION | SMART_FILTER_RECEPTION;
+	int smart = 15;
+	int receiveonly = 0;
 
 	saved_argv = argv;
 
@@ -1022,9 +1138,15 @@ lldpd_main(int argc, char *argv[])
 		case 'd':
 			debug++;
 			break;
+		case 'r':
+			receiveonly = 1;
+			break;
 		case 'm':
 			mgmtp = optarg;
 			break;
+		case 'I':
+			interfaces = optarg;
+			break;
 		case 'k':
 			advertise_version = 0;
 			break;
@@ -1066,21 +1188,7 @@ lldpd_main(int argc, char *argv[])
                         descr_override = strdup(optarg);
                         break;
 		case 'H':
-			smart = SMART_NOFILTER;
-			i = atoi(optarg);
-			if (i == 0) break;
-			if ((i < 0) || (i > 9)) {
-				fprintf(stderr, "Incorrect mode for -H\n");
-				usage();
-			}
-			if (i%3 != 0)
-				smart |= SMART_FILTER_RECEPTION;
-			if ((i + 1)%3 != 0)
-				smart |= SMART_FILTER_EMISSION;
-			if (i > 6)
-				smart |= SMART_FILTER_ONE_NEIGH | SMART_FILTER_NO_TIE;
-			if (i < 4)
-				smart |= SMART_FILTER_NO_TIE;
+			smart = atoi(optarg);
 			break;
 		default:
 			found = 0;
@@ -1099,6 +1207,14 @@ lldpd_main(int argc, char *argv[])
 				usage();
 		}
 	}
+
+	/* Set correct smart mode */
+	for (i=0; (filters[i].a != -1) && (filters[i].a != smart); i++);
+	if (filters[i].a == -1) {
+		fprintf(stderr, "Incorrect mode for -H\n");
+		usage();
+	}
+	smart = filters[i].b;
 	
 	log_init(debug, __progname);
 	tzset();		/* Get timezone info before chroot */
@@ -1119,7 +1235,12 @@ lldpd_main(int argc, char *argv[])
 		close(pid);
 	}
 
-	lsb_release = lldpd_get_lsb_release();
+	/* Try to read system information from /etc/os-release if possible.
+	   Fall back to lsb_release for compatibility. */
+	lsb_release = lldpd_get_os_release();
+	if (!lsb_release) {
+		lsb_release = lldpd_get_lsb_release();
+	}
 
 	priv_init(PRIVSEP_CHROOT);
 
@@ -1128,7 +1249,9 @@ lldpd_main(int argc, char *argv[])
 		fatal(NULL);
 
 	cfg->g_mgmt_pattern = mgmtp;
+	cfg->g_interfaces = interfaces;
 	cfg->g_smart = smart;
+	cfg->g_receiveonly = receiveonly;
 
 	/* Get ioctl socket */
 	if ((cfg->g_sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
