@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2008 Vincent Bernat <bernat@luffy.cx>
  *
- * Permission to use, copy, modify, and distribute this software for any
+ * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -102,6 +102,32 @@ struct lldpd_med_power {
 };
 #endif
 
+#ifdef ENABLE_DOT3
+#define STRUCT_LLDPD_DOT3_MACPHY "(bbww)"
+struct lldpd_dot3_macphy {
+	u_int8_t		 autoneg_support;
+	u_int8_t		 autoneg_enabled;
+	u_int16_t		 autoneg_advertised;
+	u_int16_t		 mau_type;
+};
+
+#define STRUCT_LLDPD_DOT3_POWER "(bbbbbbbbbww)"
+struct lldpd_dot3_power {
+	u_int8_t		devicetype;
+	u_int8_t		supported;
+	u_int8_t		enabled;
+	u_int8_t		paircontrol;
+	u_int8_t		pairs;
+	u_int8_t		class;
+	u_int8_t		powertype; /* If set to LLDP_DOT3_POWER_8023AT_OFF,
+					      following fields have no meaning */
+	u_int8_t		source;
+	u_int8_t		priority;
+	u_int16_t		requested;
+	u_int16_t		allocated;
+};
+#endif
+
 struct lldpd_chassis {
 	TAILQ_ENTRY(lldpd_chassis) c_entries;
 	u_int16_t		 c_refcount; /* Reference count by ports */
@@ -151,17 +177,15 @@ struct lldpd_port {
 	int			 p_id_len;
 	char			*p_descr;
 	u_int16_t		 p_mfs;
-	u_int8_t		 p_hidden; /* Hidden, this port information should
-					      be discarded if set to 1 */
+	u_int8_t		 p_hidden_in:1; /* Considered as hidden for reception */
+	u_int8_t		 p_hidden_out:2; /* Considered as hidden for emission */
 
 #ifdef ENABLE_DOT3
-#define STRUCT_LLDPD_PORT_DOT3 "lbbww"
+#define STRUCT_LLDPD_PORT_DOT3 "l" STRUCT_LLDPD_DOT3_MACPHY STRUCT_LLDPD_DOT3_POWER
 	/* Dot3 stuff */
 	u_int32_t		 p_aggregid;
-	u_int8_t		 p_autoneg_support;
-	u_int8_t		 p_autoneg_enabled;
-	u_int16_t		 p_autoneg_advertised;
-	u_int16_t		 p_mau_type;
+	struct lldpd_dot3_macphy p_macphy;
+	struct lldpd_dot3_power	 p_power;
 #else
 #define STRUCT_LLDPD_PORT_DOT3 ""
 #endif
@@ -279,12 +303,19 @@ struct protocol {
 };
 
 /* Smart mode / Hide mode */
-#define SMART_NOFILTER		0
-#define SMART_FILTER_RECEPTION	(1<<0) /* Filter received frames */
-#define SMART_FILTER_EMISSION	(1<<1) /* Filter frames to be sent */
-#define SMART_FILTER_ONE_NEIGH	(1<<2) /* Only allow one neighbor */
-#define SMART_FILTER_NO_TIE	(1<<3) /* Only allow one protocol */
-#define SMART_HIDDEN(cfg, port) ((cfg->g_smart & SMART_FILTER_RECEPTION) && port->p_hidden)
+#define SMART_INCOMING_FILTER		(1<<0) /* Incoming filtering enabled */
+#define SMART_INCOMING_ONE_PROTO	(1<<1) /* On reception, keep only one proto */
+#define SMART_INCOMING_ONE_NEIGH	(1<<2) /* On reception, keep only one neighbor */
+#define SMART_OUTGOING_FILTER		(1<<3) /* Outgoing filtering enabled */
+#define SMART_OUTGOING_ONE_PROTO	(1<<4) /* On emission, keep only one proto */
+#define SMART_OUTGOING_ONE_NEIGH	(1<<5) /* On emission, consider only one neighbor */
+#define SMART_INCOMING (SMART_INCOMING_FILTER |    \
+			 SMART_INCOMING_ONE_PROTO | \
+			 SMART_INCOMING_ONE_NEIGH)
+#define SMART_OUTGOING (SMART_OUTGOING_FILTER |		\
+			SMART_OUTGOING_ONE_PROTO |	\
+			SMART_OUTGOING_ONE_NEIGH)
+#define SMART_HIDDEN(cfg, port) ((cfg->g_smart & SMART_INCOMING_FILTER) && port->p_hidden_in)
 
 
 #define CALLBACK_SIG struct lldpd*, struct lldpd_callback*
@@ -303,6 +334,7 @@ struct lldpd {
 	time_t			 g_lastsent;
 	int			 g_lastrid;
 	int			 g_smart;
+	int			 g_receiveonly;
 #ifdef USE_SNMP
 	int			 g_snmp;
 #endif /* USE_SNMP */
@@ -313,6 +345,7 @@ struct lldpd {
 	TAILQ_HEAD(, lldpd_callback) g_callbacks;
 
 	char			*g_mgmt_pattern;
+	char			*g_interfaces;
 
         char                    *g_descr_override;
 	char			*g_lsb_release;
@@ -338,6 +371,7 @@ enum hmsg_type {
 	HMSG_SET_LOCATION,
 	HMSG_SET_POLICY,
 	HMSG_SET_POWER,
+	HMSG_SET_DOT3_POWER,
 	HMSG_SHUTDOWN
 };
 
@@ -414,6 +448,7 @@ int	 ctl_msg_pack_structure(char *, void *, unsigned int, struct hmsg *, void **
 int	 ctl_msg_unpack_structure(char *, void *, unsigned int, struct hmsg *, void **);
 
 /* interfaces.c */
+void	 lldpd_ifh_whitelist(struct lldpd *, struct ifaddrs *);
 void	 lldpd_ifh_bond(struct lldpd *, struct ifaddrs *);
 void	 lldpd_ifh_eth(struct lldpd *, struct ifaddrs *);
 #ifdef ENABLE_DOT1
